@@ -8,7 +8,7 @@ import torch
 
 class Data_Organizer(object):
     # Resampling 후 저장 추가해야 됨.
-    def __init__(self, SAVE_PATH, ID = 'ingradient', resampling = None, normalizer = None, mode = 'train', do_set_direction = False):
+    def __init__(self, SAVE_PATH, ID = 'ingradient', resampling = None, normalizer = None, mode = 'train', do_set_direction = True):
         self.crop = Cropping()
         self.SAVE_PATH = SAVE_PATH
         self.ID = ID
@@ -57,6 +57,7 @@ class Data_Organizer(object):
                 images = np.vstack(images, img)
         
 
+        print("current path : ", seg_path, "image shapes :", images.shape, "seg shapes :", seg.shape)
         img_arr, seg_arr = self.crop(images, seg)
 
         if self.do_set_direction:
